var studiox = studiox || {};
(function () {

    /* Application paths *****************************************/

    //Current application root path (including virtual directory if exists).
    studiox.appPath = studiox.appPath || '/';

    /* AUTHORIZATION **********************************************/
    //Implements Authorization API that simplifies usage of authorization scripts generated by StudioX.

    studiox.auth = studiox.auth || {};

    studiox.auth.tokenCookieName = 'StudioX.AuthToken';
    studiox.auth.tokenHeaderName = 'Authorization';

    studiox.auth.setToken = function (authToken, expireDate) {
        studiox.utils.setCookieValue(studiox.auth.tokenCookieName, authToken, expireDate, studiox.appPath);
    };

    studiox.auth.getToken = function () {
        return studiox.utils.getCookieValue(studiox.auth.tokenCookieName);
    }

    studiox.auth.clearToken = function () {
        studiox.auth.setToken();
    }

    /* UTILS ***************************************************/

    studiox.utils = studiox.utils || {};

    /**
         * Sets a cookie value for given key.
         * This is a simple implementation created to be used by StudioX.
         * Please use a complete cookie library if you need.
         * @param {string} key
         * @param {string} value
         * @param {Date} expireDate (optional). If not specified the cookie will expire at the end of session.
         * @param {string} path (optional)
         */
    studiox.utils.setCookieValue = function (key, value, expireDate, path) {
        var cookieValue = encodeURIComponent(key) + '=';

        if (value) {
            cookieValue = cookieValue + encodeURIComponent(value);
        }

        if (expireDate) {
            cookieValue = cookieValue + "; expires=" + expireDate.toUTCString();
        }

        if (path) {
            cookieValue = cookieValue + "; path=" + path;
        }

        document.cookie = cookieValue;
    };

    /**
     * Gets a cookie with given key.
     * This is a simple implementation created to be used by StudioX.
     * Please use a complete cookie library if you need.
     * @param {string} key
     * @returns {string} Cookie value or null
     */
    studiox.utils.getCookieValue = function (key) {
        var equalities = document.cookie.split('; ');
        for (var i = 0; i < equalities.length; i++) {
            if (!equalities[i]) {
                continue;
            }

            var splitted = equalities[i].split('=');
            if (splitted.length != 2) {
                continue;
            }

            if (decodeURIComponent(splitted[0]) === key) {
                return decodeURIComponent(splitted[1] || '');
            }
        }

        return null;
    };

    /**
     * Deletes cookie for given key.
     * This is a simple implementation created to be used by StudioX.
     * Please use a complete cookie library if you need.
     * @param {string} key
     * @param {string} path (optional)
     */
    studiox.utils.deleteCookie = function (key, path) {
        var cookieValue = encodeURIComponent(key) + '=';

        cookieValue = cookieValue + "; expires=" + (new Date(new Date().getTime() - 86400000)).toUTCString();

        if (path) {
            cookieValue = cookieValue + "; path=" + path;
        }

        document.cookie = cookieValue;
    }

    /* SECURITY ***************************************/
    studiox.security = studiox.security || {};
    studiox.security.antiForgery = studiox.security.antiForgery || {};

    studiox.security.antiForgery.tokenCookieName = 'XSRF-TOKEN';
    studiox.security.antiForgery.tokenHeaderName = 'X-XSRF-TOKEN';

    studiox.security.antiForgery.getToken = function () {
        return studiox.utils.getCookieValue(studiox.security.antiForgery.tokenCookieName);
    };

})();
